==> par-tasker-3.9.90-sources.jar: com/xebialabs/deployit/engine/tasker/StepExecutingActor.scala <==
package com.xebialabs.deployit.engine.tasker

import akka.actor.{ActorRef, Props, Actor, ActorLogging}
import com.xebialabs.deployit.plugin.api.flow.{ExecutionContext, StepExitCode}
import com.xebialabs.deployit.engine.api.execution.{TaskState, StepExecutionState}
import com.xebialabs.deployit.engine.api.execution.StepExecutionState._
import StepExitCode._
import com.xebialabs.deployit.engine.tasker.StepExecutingActor.messages.ExecuteStep
import java.util.UUID
import com.xebialabs.deployit.engine.tasker.messages.StepStateEventHandled
import com.xebialabs.deployit.engine.tasker.StateChangeEventListenerActor.StepStateEvent

object StepExecutingActor {
  def props(task: TaskState, ctx: NestedTaskExecutionContext) = Props(classOf[StepExecutingActor], task, ctx)

  object messages {
    case class ExecuteStep(taskStep: TaskStep, executionCtx: ExecutionContext)
  }
}

class StepExecutingActor(task: Task, ntec: NestedTaskExecutionContext) extends Actor with ActorLogging {
  def receive: Actor.Receive = {
    case ExecuteStep(s, ctx) => executeStep(s, ctx, sender)
  }

  def executeStep(step: TaskStep, ctx: ExecutionContext, sender: ActorRef) {
    val implementation = step.getImplementation
    step.setRunner()
    if (step.getState == StepExecutionState.EXECUTING) throw new IllegalStateException(s"The step [$step] is still executing, cannot run again.")
    // MDC.put(MDC_KEY_STEP_DESCRIPTION, implementation.getDescription)
    if (step.getState.isFinal) {
      log.debug(s"Will not execute: $implementation with description: ${implementation.getDescription} because it has state: ${step.getState}")
      sender ! step.getState
      return
    }
    step.recordStart()

    if (step.isMarkedForSkip) {
      setStepState(step, SKIPPED, sender)
      step.recordCompletion()
      return
    }
    setStepState(step, StepExecutionState.EXECUTING, sender)
    step.clearLog()
    log.info(s"Started $implementation")

    try {
      val result: StepExitCode = implementation.execute(ctx)
      step.recordCompletion()
      result match {
        case SUCCESS => setStepState(step, DONE, sender)
        case FAIL => setStepState(step, FAILED, sender)
        case PAUSE => setStepState(step, PAUSED, sender)
      }
    }
    catch {
      case exc: Exception =>
        ctx.logError("Step failed", exc)
        step.recordCompletion()
        setStepState(step, FAILED, sender)
      case t: Throwable =>
        ctx.logError("Step failed badly, aborting!", t)
        step.recordCompletion()
        setStepState(step, FAILED, sender)
    }
    finally {
      TaskStep.logger.info(step.getState.name)
    }
  }

  def setStepState(step: TaskStep, state: StepExecutionState, origSender: ActorRef) {
    import context._
    val oldState: StepExecutionState = step.getState
    step.setState(state)
    val taskId = task.getId
    val stepId: String = UUID.randomUUID().toString
    val waitForHandled: Boolean = state != EXECUTING
    if (waitForHandled) {
      context.system.eventStream.subscribe(this.self, classOf[StepStateEventHandled])
    }
    //    probably need a better identification of the event, ie. who is it for?
    context.system.eventStream.publish(StepStateEvent(task.getId, stepId, task, step, oldState, state, Some(ntec)))

    if (waitForHandled) {
      become({
        case StepStateEventHandled(`taskId`, `stepId`, `oldState`, `state`) =>
          log.debug(s"Handled state change [$oldState->$state]")
          become(receive)
          origSender ! state
      })
    }
  }
}

==> par-tasker-4.0.0-sources.jar: com/xebialabs/deployit/engine/tasker/StepExecutingActor.scala <==
package com.xebialabs.deployit.engine.tasker

import akka.actor.{ActorRef, Props, Actor, ActorLogging}
import com.xebialabs.deployit.plugin.api.flow.{ExecutionContext, StepExitCode}
import com.xebialabs.deployit.engine.api.execution.{TaskState, StepExecutionState}
import com.xebialabs.deployit.engine.api.execution.StepExecutionState._
import StepExitCode._
import com.xebialabs.deployit.engine.tasker.StepExecutingActor.messages.ExecuteStep
import java.util.UUID
import com.xebialabs.deployit.engine.tasker.messages.StepStateEventHandled
import com.xebialabs.deployit.engine.tasker.StateChangeEventListenerActor.StepStateEvent

object StepExecutingActor {
  def props(task: TaskState, ctx: TaskExecutionContext) = Props(classOf[StepExecutingActor], task, ctx)

  object messages {
    case class ExecuteStep(taskStep: TaskStep, executionCtx: ExecutionContext)
  }
}

class StepExecutingActor(task: Task, ntec: TaskExecutionContext) extends Actor with ActorLogging {
  def receive: Actor.Receive = {
    case ExecuteStep(s, ctx) => executeStep(s, ctx, sender)
  }

  def executeStep(step: TaskStep, ctx: ExecutionContext, sender: ActorRef) {
    val implementation = step.getImplementation
    step.setRunner()
    if (step.getState == StepExecutionState.EXECUTING) throw new IllegalStateException(s"The step [$step] is still executing, cannot run again.")
    // MDC.put(MDC_KEY_STEP_DESCRIPTION, implementation.getDescription)
    if (step.getState.isFinal) {
      log.debug(s"Will not execute: $implementation with description: ${implementation.getDescription} because it has state: ${step.getState}")
      sender ! step.getState
      return
    }
    step.recordStart()

    if (step.isMarkedForSkip) {
      step.recordCompletion()
      setStepState(step, SKIPPED, sender)
      return
    }
    setStepState(step, StepExecutionState.EXECUTING, sender)
    step.clearLog()
    log.info(s"Started $implementation")

    try {
      val result: StepExitCode = implementation.execute(ctx)
      step.recordCompletion()
      result match {
        case SUCCESS => setStepState(step, DONE, sender)
        case FAIL => setStepState(step, FAILED, sender)
        case PAUSE => setStepState(step, PAUSED, sender)
      }
    }
    catch {
      case exc: Exception =>
        ctx.logError("Step failed", exc)
        step.recordCompletion()
        setStepState(step, FAILED, sender)
      case t: Throwable =>
        ctx.logError("Step failed badly, aborting!", t)
        step.recordCompletion()
        setStepState(step, FAILED, sender)
    }
    finally {
      TaskStep.logger.info(step.getState.name)
    }
  }

  def setStepState(step: TaskStep, state: StepExecutionState, origSender: ActorRef) {
    import context._
    val oldState: StepExecutionState = step.getState
    step.setState(state)
    val taskId = task.getId
    val stepId: String = UUID.randomUUID().toString
    val waitForHandled: Boolean = state != EXECUTING
    if (waitForHandled) {
      context.system.eventStream.subscribe(this.self, classOf[StepStateEventHandled])
    }
    //    probably need a better identification of the event, ie. who is it for?
    context.system.eventStream.publish(StepStateEvent(task.getId, stepId, task, step, oldState, state, Some(ntec)))

    if (waitForHandled) {
      become({
        case StepStateEventHandled(`taskId`, `stepId`, `oldState`, `state`) =>
          log.debug(s"Handled state change [$oldState->$state]")
          become(receive)
          origSender ! state
      })
    }
  }
}
